##GWAS simulations
##simulate datasets with population structure, then run GEMMA with & without accounting for kinship
##NOTE: you need GEMMA to complete this exercise. The included GEMMA binary only works on Linux computers. 
##original code by Emily Josephs
##modified by Nancy Chen
##April 2017

##load libraries
library(MASS)
library(plyr)

##TODO: insert directory info here - replace DIR below with wherever you downloaded this folder
#setwd('DIR/QuantGenWorkshopExercises/GWAS')

##simulate data
#construct ancestral population allele freq and relatedness matrix
set.seed(12)
ancPop=runif(100, min=0, max=1) #uniform distribution of allele frequencies
#broom like Sigma
#TODO: change these values to simulate different scenarios
Sigma <- matrix(rep(0, 100) , 10, 10)
for (x in 1:5){for (y in 1:5){Sigma[x, y]=0.15} }
for (x in 6:10){for (y in 6:10){Sigma[x, y]=0.15} }
for(x in 1:10){Sigma[x, x]=0.3}
#look at relatedness matrix
Sigma
#plot 
heatmap(Sigma)

#use multivariate normal to simulate allele freq for each present-day population
presentPops1=sapply(ancPop, function(x) mvrnorm(n=1, rep(x, 10), x*(1-x)*Sigma)) #column for each locus, rows are populations. N
myBound=function(x){
if (x>1) {x=1}
if (x<0) {x=0}
 return(x)
}
presentPops=apply(presentPops1, c(1, 2), myBound) #deal with numbers greater or less than 0 (the outer bounds are sticky)

#check correlations
popCols=terrain.colors(10)

par(mfrow=c(10, 10), mar=c(0.5, 0.5, 0.5, 0.5), omi=c(0.4, 0.4, 0, 0))
for(i in 1:10){for(j in 1:10){
plot(presentPops[i, ], presentPops[j, ], ylab="", xlab="", yaxt="n", xaxt="n", bty="l", col="navyblue")
if(j==1){mtext(i, line=2, side=2, las=2, cex=2)}
if(i==10){mtext(j, line=2, side=1, cex=2)}
 } }

#simulate effect sizes, genotypes, and phenotypes for each population
#TODO: try different genetic architectures. Three options provided here, in decreasing degree of polygenicity
betas=matrix(c(rnorm(50), rep(0, 50)), ncol=1, nrow=100) #half are from a normal distribution, half are 0
save(betas, file="sims/betas1")
trueSig<-c(0:50)

betas=matrix(c(rep(0, 40), rnorm(20, mean=3, sd=0.5), rep(0, 40)), ncol=1, nrow=100) #20 are from a normal distribution with mean 3 and sd 0.5, 80 are 0 
save(betas, file="sims/betas2")
trueSig<-c(41:60)

betas=matrix(c(rep(0, 20), rep(3, 5), rep(0, 75)), ncol=1, nrow=100) #5 with beta=3, 95 are 0 
save(betas, file="sims/betas3")
trueSig<-c(21:25)

n=100 #number of loci
mym=100 #number of individuals

#table where each row is an individual (m), and columns are the # of alternate alleles at each locus (100)
getPopGenos <- function(p){
indFreqs=t(matrix(rep(presentPops[p, ], mym), nrow=n, ncol=mym))
popGenos=apply(indFreqs, c(1, 2), function(x){sum(sample(c(1, 0), 2, replace=TRUE, prob=c(x, 1-x)))})
return(popGenos)
}

pop.genos=lapply(1:10, getPopGenos) #a list of 10 elements, each is a population

#simulate phenotypes (multiply allele frequencies and effect sizes)
pop.phenos=lapply(pop.genos, function(x){x %*% betas})

#check distributions of phenotypes
par(mfrow=c(10, 1), omi=c(0, 0, 0, 0), mar=c(3, 3, 1, 1))
for(i in 1:10){hist(pop.phenos[[i]], col="dodgerblue3", breaks=20, yaxt="n", xaxt="n", border="white", main="", ylab ="", xlab="")
axis(1)
mtext(i, side=2, las=2, cex=1.5)}

#write out the files for the future
save(pop.phenos, file="sims/pop.phenos")
save(pop.genos, file="sims/pop.genos")

##TODO: run GWAS: once including kinship, once not
#write out input files for GEMMA, using BIMBAM format
gwas.set=c(1:10) #change if you only want to run GWAS on a subset of pops; for now running on all
gwas.pop.genos<-do.call(rbind, pop.genos[gwas.set])
gwas.pop.phenos<-do.call(rbind, pop.phenos[gwas.set])

#mean genotype file: rows of snps, columns of individuals
ids=sapply(1:100, function(x){paste('l', x, sep=".")})
bimbam.geno=data.frame(ids, rep('A', ncol(gwas.pop.genos)), rep('T', ncol(gwas.pop.genos)), t(gwas.pop.genos))
write.table(bimbam.geno, file="sims/test.bimbam.genotype", row.names=FALSE, col.names=FALSE, quote=FALSE, sep=", ")

#phenotype file: rows of individuals, columns of phenotypes (but here we just have one phenotype)
write.table(gwas.pop.phenos, file="sims/test.bimbam.phenotype", row.names=FALSE, col.names=FALSE, quote=FALSE)

#write out individuals to let gemma estimate kinship
set.seed(12)
myk=5000 #number of loci
mym=100
ancPop.kinship=runif(myk, min=0, max=1) #uniform distribution of allele frequencies
gemma.kinship.pops =sapply(ancPop.kinship, function(x) mvrnorm(n=1, rep(x, 10), x*(1-x)*Sigma)) #column for each locus, rows are populations. N
matrix.g=apply(gemma.kinship.pops, c(1, 2), myBound) #deal with numbers greater or less than 0 (the outer bounds are sticky)

getPopGenos <- function(p){
indFreqs=t(matrix(rep(matrix.g[p, ], mym), nrow=myk, ncol=mym)) #row for each individual, columns for locus, values are the allele freq
popGenos=apply(indFreqs, c(1, 2), function(x){sum(sample(c(1, 0), 2, replace=TRUE, prob=c(x, 1-x)))})
return(popGenos)}

pop.genosk=lapply(gwas.set, getPopGenos) #a list of 10 elements, each is a population
pop.genos1= do.call(rbind, pop.genosk) #squashing the list
ids=paste(unlist(lapply(gwas.set, function(x){rep(x, mym)})), rep(1:100, 5), sep=".")
bimbam.kinship=data.frame(ids, rep('A', ncol(pop.genos1)), rep('T', ncol(pop.genos1)), t(pop.genos1))
write.table(bimbam.kinship, file="sims/test.bimbam.kinship", row.names=FALSE, col.names=FALSE, quote=FALSE, sep=", ")

#run GEMMA
#NOTE: edit lines below if you are using your own installed version of GEMMA
#without kinship
system("./bin/gemma -g sims/test.bimbam.genotype -p sims/test.bimbam.phenotype -lm 4 -o no.kinship")

#with kinship
#estimate kinship matrix
system('./bin/gemma -g sims/test.bimbam.kinship -p sims/test.bimbam.phenotype -gk 1 -o test', wait=TRUE)
#system('./bin/gemma -g test.bimbam.genotype -p test.bimbam.phenotype -gk 1 -o test', wait=TRUE)

#check kinship matrix generated by GEMMA
gemma.kin=read.table('output/test.cXX.txt')
heatmap(as.matrix(gemma.kin))

#run GWAS
system("./bin/gemma -g sims/test.bimbam.genotype -p sims/test.bimbam.phenotype -k output/test.cXX.txt -lmm 4 -o with.kinship")

##TODO: look at results
wkin=read.table("output/with.kinship.assoc.txt", header=TRUE)
nokin=read.table("output/no.kinship.assoc.txt", header=TRUE)

#check p-values
par(mfrow=c(1, 2))
hist(nokin$p_lrt, breaks=50, col="blue", border="white", main="p values without kinship", xlab='p values', ylim=c(0, 70), xlim=c(0, 1))
hist(wkin$p_lrt, breaks=50, col="blue", border="white", main="p values with kinship", xlab='p values', ylim=c(0, 70), xlim=c(0, 1))

#QQplot
obs_nokin <- sort(nokin$p_lrt)
lobs_nokin <- -(log10(obs_nokin))

obs_wkin <- sort(wkin$p_lrt)
lobs_wkin <- -(log10(obs_wkin))

expected <- c(1:length(obs_nokin)) 
lexp <- -(log10(expected / (length(expected)+1)))
plot(c(0, 4), c(0, 4), col="blue", lwd=3, type="l", main="without kinship", xlab="Expected (-logP)", ylab="Observed (-logP)", xlim=c(0, 4), ylim=c(0, 4), las=1, xaxs="i", yaxs="i", bty="l")
points(lexp, lobs_nokin, pch=23, cex=.2, bg="black") 

plot(c(0, 4), c(0, 4), col="blue", lwd=3, type="l", main="with kinship", xlab="Expected (-logP)", ylab="Observed (-logP)", xlim=c(0, 4), ylim=c(0, 4), las=1, xaxs="i", yaxs="i", bty="l")
points(lexp, lobs_wkin, pch=23, cex=.2, bg="black") 

#check betas
hist(nokin$beta, breaks=50, col="blue", border="white", main="beta without kinship", ylim=c(0, 20), xlim=c(-5, 5))
hist(wkin$beta, breaks=50, col="blue", border="white", main="beta with kinship",  ylim=c(0, 20), xlim=c(-5, 5))

#manhattan plot
plot(-(log10(nokin$p_lrt)), xlab='SNP', ylab="-logP", main="without kinship")
rect(min(trueSig),min(-(log10(nokin$p_lrt)))-1,max(trueSig),max(-(log10(nokin$p_lrt)))+1,col=rgb(0, 0, 1, 0.4),border=rgb(0, 0, 1, 0.4))
plot(-(log10(wkin$p_lrt)), xlab='SNP', ylab="-logP", main="with kinship")
rect(min(trueSig),min(-(log10(nokin$p_lrt)))-1,max(trueSig),max(-(log10(nokin$p_lrt)))+1,col=rgb(0, 0, 1, 0.4),border=rgb(0, 0, 1, 0.4))

#note that all true significant SNPs should be in the highlighted rectangle, and all others have true beta=0. So it's clear that without kinship we get a lot of false positives
plot(nokin$beta, xlab='SNP', ylab="beta", main="without kinship")
rect(min(trueSig),min(nokin$beta)-1,max(trueSig),max(nokin$beta)+1,col=rgb(0, 0, 1, 0.4),border=rgb(0, 0, 1, 0.4))
plot(wkin$beta, xlab='SNP', ylab="beta", main="with kinship")
rect(min(trueSig),min(nokin$beta)-1,max(trueSig),max(nokin$beta)+1,col=rgb(0, 0, 1, 0.4),border=rgb(0, 0, 1, 0.4))
