##quantitative trait simulations
##simulate phenotypes given number of loci and environmental variance
##original code by Graham Coop
##modified by Nancy Chen
##April 2017

##function to simulate phenotypes given number of underlying loci (L) and environmental variance (environ.var)
pheno.geno<-function(L, environ.var){
	##Each locus is assumed to have the same allele frequency. This is just to simplify the coding, in reality these results work when each locus has its own frequency (and the coding wouldn't be too much harder). 
	allele.freq<-0.5  
	 
	#number of individuals
	Num_inds=10000
	
	##the genetic component is scaled below so that its variance is 1
	##thus you only need to play with the environmental variance 
	 
	##For each individual, at each locus we draw an allele (either 0 or 1) from the population allele frequency. 
	##We do this twice for each indivdual to represent the two haplotypes in the indivdual
	ind.hap.1<-replicate(Num_inds, rbinom(L, 1, allele.freq) )
	ind.hap.2<-replicate(Num_inds, rbinom(L, 1, allele.freq) )
	##type ind.hap.1[, 1] to see the 1st mothers 1st haplotype
	
	##Each individual's genotype at each locus is either 0, 1, 2
	ind.geno<-ind.hap.1+ind.hap.2
	
	##assuming that each allele at locus has the same effect on phenotype, sum the genotypes to get the additive genetic "phenotype"	
	if(L>1){ additive.genetic<-colSums(ind.geno)}
	if(L==1){ additive.genetic<-ind.geno}
	
	##normalize this to have variance 1, to make it comparable to the environmental variance.
	additive.genetic<-additive.genetic / sd(additive.genetic)
	
	##add the environmental contribution to the trait to get the overall phenotype
	ind.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	
	##normalize the trait to have mean zero
	ind.pheno<-ind.pheno-mean(ind.pheno)
	
	##plot a histogram of the distribution of the phenotype within the population
	#layout(1) ###done in case this is run after the code with 3 plots
	hist(ind.pheno, breaks=1000, xlab="Phenotype", main=paste("L=", L, ", VE=", environ.var, ", VA=1.", sep=""), cex.lab=1.5, cex.axis=1.5, cex.main=1.5)
}

##Many phenotypes are influenced by environmental conditions and/or many loci
##TODO: Try different values of environ.var. What happens as the environmental variance increases?
layout(t(1:3))
pheno.geno(L=1, environ.var=0)
pheno.geno(L=1, environ.var=.01)
pheno.geno(L=1, environ.var=.05)

##TODO: Try different values of L. What happens as the number of loci underlying a trait increases?
layout(t(1:3))
pheno.geno(L=1, environ.var=.05)
pheno.geno(L=4, environ.var=.05)
pheno.geno(L=10, environ.var=.05)

